#include <Adafruit_NeoPixel.h>

#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BMP085_U.h>
#include <RFduinoBLE.h>


#define PIN 6


//Adafruit_BMP085_Unified bmp = Adafruit_BMP085_Unified(10085);

int photocellPin = 2;     
int infraredPin = 3;
int acceleroPinX = 0;
int acceleroPinY = 4;
int photocellReading;


// LED strip configuration
uint8_t brightness = 100;
const int nPIXELS = 8;
const int nLEDs = nPIXELS * 3;
uint8_t ledBar[nLEDs];
const int ws2812pin = 1;


int acceleroHorizontal = 500;
int seuilVide = 800;

int etaitPlein = 0;
int mesureEffectuee = 0;
int nbVerres = 0;

int start;


void setup() {
  //Serial.begin(9600);

  // Initialise LED Strip
  pinMode(ws2812pin, OUTPUT);
  digitalWrite(ws2812pin, LOW);

  // Initialise LED infrarouge
  pinMode(infraredPin, OUTPUT);
  digitalWrite(infraredPin, LOW);

  RFduinoBLE.advertisementData = "Ecocup";
  // start the BLE stack
  RFduinoBLE.begin();
  start = millis();
  delay(1000);
  
  /* Initialise the temperature sensor */
  // Pin 5 et 6
  /*
  if(!bmp.begin()){
    Serial.print("Ooops, no BMP085 detected ... Check your wiring or I2C ADDR!");
    while(1);
  }
  */
  int i;
  nbVerres = 1;
  for(i=1;i<9;i++){
    refreshLedStrip();
    delay(100);
    nbVerres++;
  }
  nbVerres = 0;
  delay(300);
  clearStrip();
}



void loop() {  

  mesureEffectuee = 0;
  while(isHorizontal()){
    if(!mesureEffectuee){
      //Serial.println(analogRead(photocellPin));  
      int testVide = isVide();
      if(testVide == 1 && etaitPlein == 1){
        nbVerres++;
        refreshLedStrip();
        sendData();
        etaitPlein = 0;
      }
      else if (etaitPlein == 0 && testVide == 0){
        etaitPlein = 1;
      }
      mesureEffectuee = 1;
    }
  }


  /*
  mesureEffectuee = 0;
  while(isHorizontal()){
    if(mesureEffectuee == 0){
      if(isVide()){
        nbVerres++;
        refreshLedStrip();
        estPlein = 0;
      }
      mesureEffectuee = 1;
    }
  }
  */
  

  // Température 
  /*
  float temp;
  bmp.getTemperature(&temp);
 
  Serial.print("Temperature: "); Serial.print(temp); Serial.println(" C");
  delay(250);
  */
}


int isHorizontal(){
  int i;
  int sum = 0;
  int nbMesures = 5;
  int horizontal = 515;
  int y;
  
  for(i=0;i<nbMesures;i++){
    y = analogRead(acceleroPinY);
    if(y>horizontal){
      sum += y - horizontal;
    }
    else{
      sum += horizontal - y;
    }
    delay(100);
  }
  sum = sum/nbMesures;

  if(sum < 35){
    return 1;
  }
  return 0;
}


int isVide(){
  int i;
  int sum = 0;
  int nbMesures = 5;
  int error = 0.05;

  digitalWrite(infraredPin, HIGH);
  for(i=0;i<nbMesures;i++){
    sum += analogRead(photocellPin);
    delay(100);
  }
  sum = sum/nbMesures;
  digitalWrite(infraredPin, LOW);
  if(sum > seuilVide*(1-error) || sum > seuilVide*(1+error)){
    return 1;
  }
  return 0;
}


void refreshLedStrip(){
  setPixelColor(7-(nbVerres-1),Color(255/(9-nbVerres), 255/(nbVerres*2), 0));
  showStrip();
}

void sendData(){
  char data[10];
  int tps = (millis()-start)/1000;
  snprintf(data, sizeof(data), "%d\0", nbVerres);
  char *p = data+3;
  snprintf(p,sizeof(data),"%d\0",tps);
  RFduinoBLE.send(data,10);
}

void RFduinoBLE_onConnect(){
  delay(1000);
  sendData();
}

void RFduinoBLE_onReceive(char *data, int len){
  sendData();
}

// ============================
// NeoPixel API like functions:
// ============================

uint32_t Color(uint8_t r, uint8_t g, uint8_t b) {
  return ((uint32_t)r << 16) | ((uint32_t)g <<  8) | b;
}

void setPixelColor(int pixel, uint32_t color) {
  uint8_t red = (uint8_t)(color >> 16);
  uint8_t green = (uint8_t)(color >>  8);
  uint8_t blue = (uint8_t)color;
  
  red = (red * brightness) >> 8;
  green = (green * brightness) >> 8;
  blue = (blue * brightness) >> 8;
  
  setPixelColor(pixel, red, green, blue);
}

void setPixelColor(int pixel, byte red, byte green, byte blue) {
  ledBar[3*pixel] = green;
  ledBar[3*pixel+1] = red;
  ledBar[3*pixel+2] = blue;
}


void clearStrip() {
  for(uint32_t i; i<64; i++) {
    setPixelColor(i, 0);
  }
  showStrip();
}

void showStrip(){
  noInterrupts();
  for(int wsOut = 0; wsOut < nLEDs; wsOut++){
    for(int x=7; x>=0; x--){
      NRF_GPIO->OUTSET = (1UL << ws2812pin);
      if(ledBar[wsOut] & (0x01 << x)) {
        __ASM ( \
              " NOP\n\t" \
              " NOP\n\t" \
              " NOP\n\t" \
              " NOP\n\t" \
              " NOP\n\t" \
              );
        NRF_GPIO->OUTCLR = (1UL << ws2812pin);
      
      }else{
        NRF_GPIO->OUTCLR = (1UL << ws2812pin);
        __ASM ( \
              " NOP\n\t" \
              " NOP\n\t" \
              " NOP\n\t" \
              );      
      }
    }
  }
  delayMicroseconds(50); // latch and reset WS2812.
  interrupts();  
}
    

